﻿/*
 * Object for holding item data from TOC tree
 */
function TocDataObject(type, category, level, url, id, text, baselevel, figure, table, code)
{
  // Parameters (do not change after generation)
  this.dataType = type;
  this.dataCategory = category;
  this.dataLevel = level;
  this.dataURL = url;
  this.dataID = id;
  this.dataText = text;
  // Heading number and figure number control
  this.dataBaselevel = baselevel;
  this.dataFigureNo = figure;
  this.dataTableNo = table;
  this.dataCodeNo = code;
  // For list display
  this.expand = false;
  this.viewIcon = true;
  this.dataQuery = '';
  this.dataLabel = '';
  this.dataExpandLabel = '';
}

TocDataObject.prototype = {
  getType       : function () {return this.dataType;},
  getCategory   : function () {return this.dataCategory;},
  getLevel      : function () {return this.dataLevel;},
  getURL        : function () {return this.dataURL;},
  getID         : function () {return this.dataID;},
  getText       : function () {return this.dataText;},
  getBaselevel  : function () {return this.dataBaselevel;},
  getFigureNo   : function () {return this.dataFigureNo;},
  getTableNo    : function () {return this.dataTableNo;},
  getCodeNo     : function () {return this.dataCodeNo;},

  enableExpand    : function ()     {this.expand = true;},
  isExpandFolder  : function ()     {return this.expand;},
  setQuery        : function (str)  {this.dataQuery = ((str == null) ? '' : str);},
  getQuery        : function ()     {return this.dataQuery;},
  setFigureNo     : function (no)   {this.dataFigureNo = (((no == null) || (no < 0)) ? 0 : no);},
  setTableNo      : function (no)   {this.dataTableNo = (((no == null) || (no < 0)) ? 0 : no);},
  setCodeNo       : function (no)   {this.dataCodeNo = (((no == null) || (no < 0)) ? 0 : no);},
  setLabel        : function (str)  {this.dataLabel = ((str == null) ? '' : str);},
  getLabel        : function ()     {return this.dataLabel; },
  setExpandLabel  : function (str)  {this.dataExpandLabel = ((str == null) ? '' : str);},
  getExpandLabel  : function ()     {return this.dataExpandLabel;},
  enableViewIcon  : function ()     {this.viewIcon = true;},
  disableViewIcon : function ()     {this.viewIcon = false;},
  isViewIcon      : function ()     {return this.viewIcon;}
};

/*
 * Overall TOC
 */
function TocObject()
{
  // Logo Data
  this.logoFile = '';
  this.logoWidth = 0;
  this.logoHeight = 0;
  // Title
  this.titleName = '';
  this.titleFile = TemplateSettings.templateTitleFileName;
  this.windowTitleName = '';
  // Display Settings
  this.displayNumber = false;
  this.modeAnchor = true;
  this.targetName = '';
  this.windowWidth = 280;
  this.openWithExpand = false;
  this.viewFileIcon = true;
  this.printPage = false;
  this.insertTitlePage = false;
  // Auto-sequence control
  this.baselevel = 0;
  this.figureNo = 0;
  this.tableNo = 0;
  this.codeNo = 0;
  // For Guidelines
  this.currentURL = '';
  this.countGuideline = 1;
  this.rootURL = '';
  // Find
  this.searchEnabled = true;
  // List Data
  this.arrayData = new Array();
  this.arrayData.length = 0;

  // Logo file
  this.setLogoFile = function (file)
  {
    this.logoFile = ((file == null) ? '' : file);
  }
  this.getLogoFile = function ()
  {
    return this.logoFile;
  }

  // Logo size
  this.setLogoSize = function (width, height)
  {
    this.logoWidth = (((width == null) || (width < 0)) ? 0 : width);
    this.logoHeight = (((height == null) || (height < 0)) ? 0 : height);
  }
  this.getLogoWidth = function ()
  {
    return this.logoWidth;
  }
  this.getLogoHeight = function ()
  {
    return this.logoHeight;
  }

  // Document title
  this.setTitle = function (title)
  {
    this.titleName = ((title == null) ? '' : title);
  }
  this.getTitle = function ()
  {
    return this.titleName;
  }

  // Window title
  this.setWindowTitle = function (title)
  {
    this.windowTitleName = ((title == null) ? '' : title);
  }
  this.getWindowTitle = function ()
  {
    return this.windowTitleName;
  }

  // Title page (top page)
  this.setTitleFile = function (file)
  {
    this.titleFile = ((file == null) ? '' : file);
  }
  this.getTitleFile = function ()
  {
    return this.titleFile;
  }

  // Target frame name
  this.setTargetName = function (name)
  {
    this.targetName = ((name == null) ? '' : name);
  }
  this.getTargetName = function ()
  {
    return this.targetName;
  }
  this.isEnableTarget = function ()
  {
    return (this.targetName != '');
  }

  // Window width initial setting
  this.setWindowWidth = function (width)
  {
    this.windowWidth = (((width == null) || (width < 0)) ? 0 : width);
  }
  this.getWindowWidth = function ()
  {
    return this.windowWidth;
  }

  // Heading number display
  this.enableDisplayNumber = function ()
  {
    this.displayNumber = true;
  }
  this.disableDisplayNumber = function ()
  {
    this.displayNumber = false;
  }
  this.isDisplayNumber = function ()
  {
    return this.displayNumber;
  }

  // Tree initial setting
  this.setOpenWithExpand = function (enable)
  {
    this.openWithExpand = enable;
  }
  this.isOpenWithExpand = function ()
  {
    return this.openWithExpand;
  }

  // Heading number base level
  this.setBaselevel = function (level)
  {
    this.baselevel = (((level == null) || (level < 0)) ? 0 : level);
  }
  this.getBaselevel = function ()
  {
    return this.baselevel;
  }

  // Figure number initial value (figures)
  this.setFigureNo = function (no)
  {
    this.figureNo = (((no == null) || (no < 0)) ? 0 : no);
  }
  this.getFigureNo = function ()
  {
    return this.figureNo;
  }
  this.addFigureNo = function (num)
  {
    var no = parseInt(num);
    if (isNaN(num)) no = 1;
    this.figureNo += no;
  }

  // Figure number initial value (tables)
  this.setTableNo = function (no)
  {
    this.tableNo = (((no == null) || (no < 0)) ? 0 : no);
  }
  this.getTableNo = function ()
  {
    return this.tableNo;
  }
  this.addTableNo = function (num)
  {
    var no = parseInt(num);
    if (isNaN(num)) no = 1;
    this.tableNo += no;
  }

  // Figure number initial value (code)
  this.setCodeNo = function (no)
  {
    this.codeNo = (((no == null) || (no < 0)) ? 0 : no);
  }
  this.getCodeNo = function ()
  {
    return this.codeNo;
  }
  this.addCodeNo = function (num)
  {
    var no = parseInt(num);
    if (isNaN(num)) no = 1;
    this.codeNo += no;
  }

  // Anchor tag mode (normal/GET)
  this.setAnchorMode = function ()
  {
    this.modeAnchor = true;
  }
  this.setGetMode = function ()
  {
    this.modeAnchor = false;
  }
  this.isAnchorMode = function ()
  {
    return this.modeAnchor;
  }
  this.isGetMode = function ()
  {
    return !(this.modeAnchor);
  }

  // Non-closing list icon
  this.enableViewFileIcon = function ()
  {
    this.viewFileIcon = true;
  }
  this.disableViewFileIcon = function ()
  {
    this.viewFileIcon = false;
  }

  // Display printable page
  this.enablePrintPage = function ()
  {
    this.printPage = true;
  }
  this.disablePrintPage = function ()
  {
    this.printPage = false;
  }
  this.isEnablePrintPage = function ()
  {
    return this.printPage;
  }

  // Inserts title page at start of the printable page
  this.setInsertTitlePage = function (flag)
  {
    this.insertTitlePage = flag;
  }
  this.isEnableInsertTitlePage = function ()
  {
    return this.insertTitlePage;
  }

  // Root URL
  this.setRootURL = function (url)
  {
    this.rootURL = ((url == null) ? '' : url);
  }
  this.getRootURL = function ()
  {
    return this.rootURL;
  }

  // Find
  this.setSearchEnabled = function (flag)
  {
    this.searchEnabled = flag;
  }
  this.isEnabledSearch = function ()
  {
    return this.searchEnabled;
  }

  // Data array operations
  this.getDataArray = function ()
  {
    return this.arrayData;
  }
  this.getDataCount = function ()
  {
    return this.arrayData.length;
  }
  this.getData = function (no)
  {
    if ((no < 0) || (no >= this.getDataCount())) return null;
    return this.arrayData[no];
  }

  this._addTocDataObjectCore = function (type, category, level, name, url, id)
  {
    if (level <= this.baselevel) {this.figureNo = 1; this.tableNo = 1; this.codeNo = 1;}
    var data = new TocDataObject(type, category, level, url, id, name, this.baselevel, this.figureNo, this.tableNo, this.codeNo);
    this.arrayData.push(data);
  }

  // Heading numbering (emphasis)
  this.addSectionHeader = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('MAN_HEADER', category, level, name, url, id);
  }

  // Heading numbering
  this.addSection = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('MAN', category, level, name, url, id);
  }

  // No heading number (emphasis)
  this.addHeader = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('HEADER', category, level, name, url, id);
  }

  // No heading number
  this.addItem = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('ITEM', category, level, name, url, id);
  }

  // Guideline (emphasis) (no number)
  this.addGuidelineSection = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('GUIDELINE_SECTION', category, level, name, url, id);
    this.currentURL = url;
    this.countGuideline = 1;
  }

  // Guideline (emphasis)
  this.addGuidelineHeader = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('GUIDELINE_HEADER', category, level, name, url, id);
    this.currentURL = url;
    this.countGuideline = 1;
  }

  // Guideline background
  this.addGuidelineDesc = function (category, level, name, url, id)
  {
    this._addTocDataObjectCore('GUIDELINE', category, level, name, url, id);
    this.currentURL = url;
    this.countGuideline = 1;
  }

  // Guideline Item
  this.addGuidelineItem = function (category, level, name)
  {
    if (level <= this.baselevel) {this.figureNo = 1; this.tableNo = 1; this.codeNo = 1;}
    var data = new TocDataObject('GUIDELINE', category, level, this.currentURL, TemplateSettings.guidelineItemAnchorPrefix + this.countGuideline, name, this.baselevel, this.figureNo, this.tableNo, this.codeNo);
    data.disableViewIcon();
    this.arrayData.push(data);
    this.countGuideline++;
  }

  // Separator line
  this.addSeparator = function ()
  {
    this.figureNo = 1; this.tableNo = 1; this.codeNo = 1;
    var data = new TocDataObject('SEPARATOR', '', 0, '', '', '', this.figureNo, this.tableNo, this.codeNo);
    this.arrayData.push(data);
  }

  // Blank line
  this.addBlankLine = function ()
  {
    this.figureNo = 1; this.tableNo = 1; this.codeNo = 1;
    var data = new TocDataObject('BLANK', '', 0, '', '', '', this.figureNo, this.tableNo, this.codeNo);
    this.arrayData.push(data);
  }

  // External file
  this.addExternal = function (category, level, name, url)
  {
    this._addTocDataObjectCore('EXTERNAL', category, level, name, url, '');
  }

  // Anchor tag generation (normal)
  this.makeAnchorURL = function (data)
  {
    if (data == null) return '';
    var level = '';
    var label = '';
    var url = '';
    var text = '';
    if (data.getType().match(/^man|^guideline/i))
    {
      level = '?level=' + data.getQuery() + '&figure=' + data.getFigureNo() + '&table=' + data.getTableNo() + '&code=' + data.getCodeNo();
      label = data.getLabel();
    }
    url = data.getURL();
    text = ((this.isDisplayNumber() && (label != '')) ? (label + ' ') : '') + data.getText();
//    text = ((this.isDisplayNumber()) ? label : '') + data.getText();
    if (url == '')
    {
      url = text;
    }
    else 
    {
      var target = (this.isEnableTarget() ? (' target="' + this.getTargetName() +'"') : '');
      var id = ((data.getID() == '') ? '' : ('#' + data.getID()));
      if (data.getType().match(/^external/i))
      {
        url = '<a href="' + this.rootURL + 'external.html?file=' + url + '"' + target + '>' + text + '</a>';
      }
      else
      {
        url = '<a href="' + this.rootURL + url + level + id + '"' + target + '>' + text + '</a>';
      }
    }
    return url;
  }

  // Anchor tag generation (GET)
  this.makeGetURL = function (data)
  {
    if (data == null) return '';
    var url = '';
    var text = '';
    var label = '';
    var level = '';
    var figure = '';
    var table = '';
    var code = '';
    if (data.getType().match(/^man|^guideline/i))
    {
      level = data.getQuery();
      label = data.getLabel();
      figure = data.getFigureNo();
      table = data.getTableNo();
      code = data.getCodeNo();
    }
    url = data.getURL();
    var displayNumber = ((data.getType().match(/section/i)) ? false : this.isDisplayNumber());
    text = ((this.isDisplayNumber() && (label != '')) ? (label + ' ') : '') + data.getText();
    if (url == '')
    {
      url = text;
    }
    else 
    {
      var target = (this.isEnableTarget() ? this.getTargetName() : '');
      var id = ((data.getID() == '') ? '' : ('#' + data.getID()));
      if (data.getType().match(/^external/i))
      {
        url = '<a href="javascript: Guideline.viewPage(\'' + this.rootURL + 'external.html?file=' + url + '\',\'' + target + '\',\'1\',1,1,1);">' + text + '</a>';
      }
      else
      {
        url = '<a href="javascript: Guideline.viewPage(\'' + this.rootURL + url + id + '\',\'' + target + '\',\'' + level + '\',' + figure + ',' + table + ',' + code + ');">' + text + '</a>';
      }
    }
    return url;
  }

  // Tree data processing
  this.processTreeData = function ()
  {
    var prevURL = '';
    // Heading open/close check
    var arrayLevel = new Array(9);
    for (var x = 0; x < 9; x++) arrayLevel[x] = 0;
    var prevLevel = 999;
    var prevData = null;
    for (var i = 0; i < this.getDataCount(); i++)
    {
      var label = '';
      var query = '';
      var expandLabel = '';
      var data = this.getData(i);
      var currentLevel = data.getLevel();
      var baselevel = data.getBaselevel();
      var dataType = data.getType();
      var currentURL = data.getURL();
      if (currentLevel > prevLevel)
      {
        if (i > 0) this.getData(i - 1).enableExpand();
      }
      arrayLevel[currentLevel] = arrayLevel[currentLevel] + 1;
      for (var n = (currentLevel + 1); n < 9; n++) arrayLevel[n] = 0;
      if (dataType.match(/separator|blank/i))
      {
        // Temporarily reset hierarchy level to 0 (should have no effect on ID because it does not open and close)
        arrayLevel[0] = arrayLevel[0] - 1;
        if (arrayLevel[0] < 0) arrayLevel[0] = 0;
      }
      for (var n = 0; n < 9; n++)
      {
        if (arrayLevel[n] > 0)
        {
          expandLabel += (((n > 0) ? '_' : '') + arrayLevel[n]);
          var x = n - baselevel;
          if (x >= 0)
          {
            query += (((x > 0) ? ',' : '') + arrayLevel[n]);
            if (x < 4)
            {
//              label += (((x > 0) ? '.' : '') + arrayLevel[n]);
              label += (arrayLevel[n] + '.');
            }
            else
            {
              label = '';
            }
          }
        }
      }
      // alert('label="' + label + '" query="' + query + '" expand="' + expandLabel + '"');
      // Use same parameters for sending if this has the same URL as the previous TOC item
      if ((currentURL == prevURL) && (currentURL != '') && (i > 0))
      {
        // data.setQuery(this.getData(i - 1).getQuery());
        data.setQuery(prevData.getQuery());
        data.setFigureNo(prevData.getFigureNo());
        data.setTableNo(prevData.getTableNo());
        data.setCodeNo(prevData.getCodeNo());
      }
      else
      {
        data.setQuery(query);
        prevData = data;
      }
      data.setLabel(label);
      data.setExpandLabel(expandLabel);
      prevLevel = currentLevel;
      prevURL = currentURL;
    }
  }

  // Tree generation
  this.createContents = function ()
  {
    // Display logo data
    var file = this.getLogoFile();
    if (file != '')
    {
      var elem = document.getElementById(TemplateSettings.titleLogoDivId);
      if (elem != null)
      {
        var obj = document.createElement('IMG');
        obj.src = this.rootURL + file;
        var width = this.getLogoWidth();
        var height = this.getLogoHeight();
        if (width > 0) obj.width = width;
        if (height > 0) obj.height = height;
        elem.appendChild(obj);
      }
    }

    // Display title
    var title = this.getTitle();
    {
      var elem = document.getElementById(TemplateSettings.titleTocDivId);
      if (elem != null)
      {
        var obj = document.createElement('A');
        obj.href = this.rootURL + this.getTitleFile();
        if (this.isEnableTarget()) obj.target = this.getTargetName();
        obj.innerHTML = title;
        elem.appendChild(obj);
      }
    }

    // Display search
    if (this.isEnabledSearch())
    {
      var elem = document.getElementById(TemplateSettings.searchBoxMiniDivId);
      if (elem != null)
      {
        elem.className = TemplateSettings.searchBoxMiniDivStyle;
        var obj = document.createElement('FORM');
        obj.name = TemplateSettings.searchBoxMiniFormName;
        obj.action = this.rootURL + TemplateSettings.searchFileName;
        obj.method = 'GET';
        var formHtml = '';
        formHtml += '<input type="text" name="q" class="' + TemplateSettings.searchBoxMiniTextStyle + '"> ';
        formHtml += '<input type="submit" class="' + TemplateSettings.searchBoxMiniButtonStyle + '" value="' + LocaleString.lcStrLabelStartSearch + '">';
        obj.innerHTML = formHtml;
        elem.appendChild(obj);
      }
    }

    // Change window title
    {
      if (this.getWindowTitle() != '')
      {
        document.title = this.getWindowTitle();
      }
      else if (document.title == '')
      {
        document.title = this.getTitle();
      }
    }

    // Control of printable page
    {
      var elem = document.getElementById(TemplateSettings.printControllerId);
      if (elem != null)
      {
        var str = '';
        str += '<select id="' + TemplateSettings.selectCategoryId + '" name="category" size="1" class="' + TemplateSettings.styleSelectCategory + '" onChange="Reassemble.viewPrintPage();">' +
               '<option value="">' + LocaleString.lcStrLabelSelectChapter + '</option>' +
               '</select><br />';
        if (this.isEnablePrintPage())
        {
          elem.innerHTML += str;
          elem.style.display = 'block';
          var selectObj = document.getElementById(TemplateSettings.selectCategoryId);
          if (selectObj != null)
          {
            for (var i = 0; i < this.getDataCount(); i++)
            {
              var data = this.getData(i);
              if ((data != null) && (data.getType().match(/section/i)))
              {
                var optObj = document.createElement('OPTION');
                optObj.innerHTML = ((data.getType().match(/^man|^guideline/i) && this.isDisplayNumber() && (data.getLabel() != '')) ? (data.getLabel() + ' ') : '') + data.getText();
                optObj.value     = data.getCategory();
                selectObj.appendChild(optObj);
              }
            }
            var optObj = document.createElement('OPTION');
            optObj.innerHTML = LocaleString.lcStrLabelSelectAll;
            optObj.value     = '*';
            selectObj.appendChild(optObj);
          }
        }
      }
    }

    // Display tree
    var prev = 0;
    var objDIVs = new Array(9);
    for (var x = 0; x < 9; x++) objDIVs[x] = null;
    var rootObj = document.getElementById(TemplateSettings.divTreePaneId);
    if (rootObj != null)
    {
      objDIVs[0] = rootObj;
      for (var i = 0; i < this.getDataCount(); i++)
      {
        var data = this.getData(i);
        if (data == null) continue;
        var current = data.getLevel();
        var dataType = data.getType();
        var dataLabel = data.getLabel();
        var dataExpandLabel = data.getExpandLabel();
//        alert('i=' + i + ' current=' + current + ' prev=' + prev);
        // Close previous DIVs
        if (current < prev)
        {
          for (var x = prev; x > current; x--)
          {
            objDIVs[x - 1].appendChild(objDIVs[x]);
          }
        }

        // Add list
        var objDIV = document.createElement('DIV');
        objDIV.className = ExpandFolder.styleLineIconDiv;
        if (dataType.match(/^separator$/i))
        {
          objDIV.innerHTML = '<hr class="' + TemplateSettings.styleSeparatorLine + '">';
        }
        else if (dataType.match(/^blank$/i))
        {
          objDIV.innerHTML = '<br>';
        }
        else
        {
          var objSPAN = document.createElement('SPAN');
          if (dataType.match(/header|section/i))
          {
            objSPAN.className = TemplateSettings.styleHeaderLineSpan;
          }
          else
          {
            objSPAN.className = TemplateSettings.styleItemLineSpan;
          }
          if (this.modeAnchor)
          {
            objSPAN.innerHTML = this.makeAnchorURL(data);
          }
          else
          {
            if (dataType.match(/^man|^guideline/i))
            {
              objSPAN.innerHTML = this.makeGetURL(data);
            }
            else
            {
              objSPAN.innerHTML = this.makeAnchorURL(data);
            }
          }
          var objIMG = document.createElement('IMG');
          if (data.isExpandFolder())
          {
            objIMG.src = (this.isOpenWithExpand() ?  ExpandFolder.openIconPath : ExpandFolder.closeIconPath);
            objIMG.className = (this.isOpenWithExpand() ? ExpandFolder.styleOpenImg : ExpandFolder.styleCloseImg);
            objIMG.onclick = new Function('ExpandFolder.expand(\'' + ExpandFolder.expandDivIdPrefix + data.getExpandLabel() + '\', this);');
            objIMG.id = (ExpandFolder.expandImgIdPrefix + ExpandFolder.expandDivIdPrefix + data.getExpandLabel());
            objDIV.appendChild(objIMG);
          }
          else
          {
            if (!(this.viewFileIcon) || !(data.isViewIcon()))
            {
              objDIV.className = ExpandFolder.styleLineNoIconDiv;
            }
            else
            {
              objIMG.src = ExpandFolder.fileIconPath;
              objIMG.className = ExpandFolder.styleFileImg;
              objDIV.appendChild(objIMG);
            }
          }
          objDIV.appendChild(objSPAN);
        }
        objDIVs[current].appendChild(objDIV);

        // DIV control
        if (data.isExpandFolder())
        {
          var objDIV = document.createElement('DIV');
          objDIV.className = (this.isOpenWithExpand() ? ExpandFolder.styleOpenDiv : ExpandFolder.styleCloseDiv);
          objDIV.id = ExpandFolder.expandDivIdPrefix + data.getExpandLabel();
          objDIV.setAttribute(ExpandFolder.attributeFolded, !(this.isOpenWithExpand()));
          objDIVs[current + 1] = objDIV;
        }
        prev = current;
      }
      for (var x = prev; x > 0; x--)
      {
        objDIVs[x - 1].appendChild(objDIVs[x]);
      }
    }
  }
}

